#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# IMPORTS
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

from __future__ import annotations;

from src.thirdparty.types import *;
from src.thirdparty.maths import *;

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# EXPORTS
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

__all__ = [
    'randomset_integers',
    'randomset_alphabet',
    'randomset_greek',
    'random_function',
];

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# CONSTANTS / VARIABLES
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

ALPHA = 'abcdefghijklmnopqrstuvwxyz';
GREEK = 'αβγδεζηθικλμνξοπρςτυφχψω';
T1 = TypeVar('T1');
T2 = TypeVar('T2');

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# METHODS
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

def randomset_integers(low: int, high: int) -> list[int]:
    N = random.randint(low, high);
    return list(range(1, N+1));

def randomset_alphabet(low: int, high: int) -> list[int]:
    N = random.randint(low, high);
    return list([a for k, a in enumerate(ALPHA) if k < N]);

def randomset_greek(low: int, high: int) -> list[int]:
    N = random.randint(low, high);
    return list([a for k, a in enumerate(GREEK) if k < N]);

def random_function(
    X: list[T1],
    Y: list[T2],
    injective: Optional[bool] = None,
    surjective: Optional[bool] = None,
) -> list[tuple[T1, T2]]:
    # TODO: add feature to force injectivity/surjectivity, if possible.
    # m = len(X);
    # n = len(Y);
    # if m > n:
    #     injective = False;
    # if m < n:
    #     surjective = False;
    return [ (x, random.choice(Y)) for x in X ];
